#!/bin/bash 

# https://docs.microsoft.com/en-us/azure/active-directory/develop/howto-create-service-principal-portal
FUNCTION_HOST_KEY_APP_ID="8e318f27-9c42-43d8-bdce-5e0f0b935094"
TENANT="9c02fa12-5035-4da6-a43c-a0f77824195c"
SUBSCRIPTION_ID="05de1293-e942-45b4-bce2-b8dd3ecac227"
LOG_FILE="function-host-key.log"

echo "" > $LOG_FILE

RESOURCE_GROUP=$1
if [ -z "$RESOURCE_GROUP" ]; then
  echo "No resource group namefound as first param"
  exit 1
fi

FUNCTION_APP_NAME=$2
if [ -z "$FUNCTION_APP_NAME" ]; then
  echo "No function app id found as the second param"
  exit 1
fi

KEY_NAME=$3
if [ -z "$KEY_NAME" ]; then
  echo "No key name found as the third param"
  exit 1
fi

API_URL="https://$FUNCTION_APP_NAME.scm.azurewebsites.net/api/functions/admin/token"
SITE_URL="https://$FUNCTION_APP_NAME.azurewebsites.net/admin/host/keys/$KEY_NAME"

### Grab a fresh bearer access token.
ACCESS_TOKEN=$(curl -s -X POST -F grant_type=client_credentials -F resource=https://management.azure.com/ -F client_id=$FUNCTION_HOST_KEY_APP_ID -F client_secret=$FUNCTION_HOST_KEY_APP_SECRET https://login.microsoftonline.com/$TENANT/oauth2/token | jq '.access_token' -r)
echo $ACCESS_TOKEN >> $LOG_FILE
if [[ "$ACCESS_TOKEN" == "null" ]] ; then
  echo "Wrong FUNCTION_HOST_KEY_APP credentials"
  exit 1
fi

### Grab the publish data for the Funciton App and output it to an XML file.
PUBLISH_DATA=$(curl -s -X POST -H "Content-Length: 0" -H "Authorization: Bearer $ACCESS_TOKEN" https://management.azure.com/subscriptions/$SUBSCRIPTION_ID/resourceGroups/$RESOURCE_GROUP/providers/Microsoft.Web/sites/$FUNCTION_APP_NAME/publishxml?api-version=2016-08-01)
echo $PUBLISH_DATA > publish_data.xml
echo $PUBLISH_DATA >> $LOG_FILE

### Grab the Kudu username and password from the publish data XML file.
USER_NAME=$(xmlstarlet sel -t -v "//publishProfile[@profileName='$FUNCTION_APP_NAME - Web Deploy']/@userName" publish_data.xml)
USER_PASSWORD=$(xmlstarlet sel -t -v "//publishProfile[@profileName='$FUNCTION_APP_NAME - Web Deploy']/@userPWD" publish_data.xml)

### Generate a JWT that can be used with the Functions Key API.
JWT=$(curl -s -X GET -u $USER_NAME:$USER_PASSWORD $API_URL | tr -d '"')
echo "JWT: $JWT" >> $LOG_FILE

### Grab the key from the Functions Key API.
KEY=$(curl -s -X POST -H "Content-Length: 0" -H "Authorization: Bearer $JWT" $SITE_URL | jq -r '.value') 
echo "$KEY"
