'use strict'

const express = require('express')
const azureFunctionExpress = require('azure-function-express')

// we use ts-node to run "messages" function during development, so we need to import source src/*.ts files instead of dist/*.js
// (use eval to prevent funcpack's webpack execution from unnecessasry src folder require)
const nodeSdk =
  process.env.NODE_ENV === 'development'
    ? eval('require')('@dvorakjan/feedbot-node-sdk/src')
    : require('@dvorakjan/feedbot-node-sdk/dist/node-sdk/src')

const plugins = nodeSdk.Plugins

// can use .env file during development
require('dotenv').config()

const app = express()

// npm package cors doesn't work - need to do it by hand
const allowCrossDomain = function(req, res, next) {
  res.header('Access-Control-Allow-Origin', '*')
  res.header('Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE,OPTIONS')
  res.header(
    'Access-Control-Allow-Headers',
    'Content-Type, Authorization, Content-Length, X-Requested-With, Accept'
  )

  // intercept OPTIONS method
  if ('OPTIONS' == req.method) {
    res.send(200)
  } else {
    next()
  }
}
app.use(allowCrossDomain)

for (const id in plugins) {
  plugins[id].registerManagementRoutes(app, '/api/management')
}
app.all('*', (req, res) => res.send(404))

module.exports = azureFunctionExpress.createHandler(app)
