const winston = require('winston')
const chalk = require('chalk')

// logger
const loggerFormat = winston.format.printf(info => {
  let levelMessage, textMessage
  switch (info.level) {
    case 'error':
      levelMessage = chalk.red(info.level.toUpperCase())
      textMessage = chalk.redBright(info.message)
      break
    case 'info':
      levelMessage = chalk.white(info.level.toUpperCase())
      textMessage = chalk.white(info.message)
      break
    case 'warning':
      levelMessage = chalk.yellow(info.level.toUpperCase())
      textMessage = chalk.yellowBright(info.message)
      break
    case 'notice':
      levelMessage = chalk.green(info.level.toUpperCase())
      textMessage = chalk.greenBright(info.message)
      break
    default:
      levelMessage = chalk.white(info.level.toUpperCase())
      textMessage = chalk.white(info.message)
      break
  }
  return (
    chalk.grey(info.timestamp) +
    chalk.italic(' [Azure-Function:' + info.label + '] ') +
    levelMessage +
    ': \n' +
    textMessage
  )
})

module.exports = function(label) {
  return winston.createLogger({
    levels: winston.config.syslog.levels,
    format: winston.format.combine(
      winston.format.label({ label: label }),
      winston.format.timestamp(),
      loggerFormat
    ),
    transports: [new winston.transports.Console()]
  })
}
